#!/usr/bin/perl -w
#
# Copyright 2001 Sun Microsystems, Inc.  All rights reserved.
# $Id: process_killer.pl,v 1.1.2.3 2001/07/20 04:39:06 asun Exp $
#
# Look for run-away httpd and caspeng processes.  Restart caspeng.
# Kill httpd.

use strict;
use Cobalt::Product;

# only bother if the 5 minute load average is above 0.91
`/usr/bin/uptime` =~ /: [\d\.]+, ([\d\.]+), [\d\.]+$/;

if ($1 <= 0.91)
{
    exit(0);
}

# something may be amiss, check with ps
my @httpds = get_proc_info('httpd');

my @httpd_kills = ();
my @httpd_possibly = ();

# note: this process actually affects the system enough to lower the
#       cpu utilization of a runaway httpd process. as a result, we
#       we use much lower numbers than expected. 
for my $httpd (@httpds)
{
    my ($pid, $pcpu, $htime) = split(' ', $httpd);

    $htime =~ /(\d+):(\d+):(\d+)/;
    my $time = ($1*60 + $2)*60 + $3;
    next unless $time gt 600;
    
    if ($pcpu > 77)
    {
        # everything over 77% cpu gets killed
        push @httpd_kills, $pid;
    }
    elsif ($pcpu > 38)
    {
        # could be a runaway httpd combined with a runaway caspeng
        push @httpd_possibly, $pid;
    }
}

my @caspengs = get_proc_info('caspeng');
@caspengs = (@caspengs, get_proc_info('caspd'));

my $restart_caspeng = 0;
my $kill_possibles = 0;

for my $caspeng (@caspengs)
{
    my ($pid, $pcpu, $htime) = split(' ', $caspeng);

    $htime =~ /(\d+):(\d+):(\d+)/;
    my $time = ($1*60 + $2)*60 + $3;
    next unless $time gt 600; # 10 minutes of cpu time

    if ($pcpu > 77)
    {
        $restart_caspeng = 1;
        last;
    }
    elsif ($pcpu > 38 && scalar(@httpd_possibly))
    {
        $restart_caspeng = 1;
        $kill_possibles = 1;
        last;
    }
}

# if there is a combo httpd/caspeng loop push the possibles onto the kill 
# list
if ($kill_possibles)
{
    push @httpd_kills, @httpd_possibly;
}

# we probably don't want to just kill the parent apache processes
# do a restart for those if they are offenders, but they should 
# never do much so this saftey check is probably unnecessary
my $adm_pid = get_pid($AdminPid);
my $httpd_pid = get_pid($Htpid);

my $restart_adm = 0;
my $restart_httpd = 0;

if (grep(/^$adm_pid$/, @httpd_kills))
{
    $restart_adm = 1;
}
if (grep(/^$httpd_pid$/, @httpd_kills))
{
    $restart_httpd = 1;
}

# actually do what needs to be done
# do the kill first, so we don't try to kill processes that no longer
# exist if a full restart is needed

# kill individual httpd processes
if (scalar(@httpd_kills))
{
    log_msg('Sending SIGTERM to run away httpd processes: ' 
            . join(', ', @httpd_kills));
    kill 15, @httpd_kills;
}

# restart admserv if the parent was a run away
if ($restart_adm)
{
    log_msg('Had to restart run away admserv process!');
    system('/etc/rc.d/init.d/admserv restart &>/dev/null');
}

# restart httpd if the parent was a run away
if ($restart_httpd)
{
    log_msg('Had to restart run away httpd process!');
    system('/etc/rc.d/init.d/httpd restart &>/dev/null');
}

# restart the caspeng
if ($restart_caspeng)
{
    log_msg('Had to restart run away caspeng process!');
    system('/etc/rc.d/init.d/asp-apache-3000 restart &>/dev/null');
}

exit(0);

# helper routines
sub get_proc_info
{
    my $name = shift;
    my @info = `/bin/ps --no-headers -w -o pid,%cpu,time -C $name`;
    chomp(@info);
    
    return @info;
}

sub get_pid
{
    my $pid_file = shift;
    open(PID, $pid_file) or return(-1);
    my $pid = <PID>;
    close(PID);

    $pid =~ /(\d+)/;
    return $1;
}

# log to syslog, so it's obvious something was done
sub log_msg
{
    my $msg = shift;

    # need the newline or logger hangs, apparently it looks
    # for a new line to terminate input
    system("/usr/bin/logger -p daemon.notice -t swatch $msg\n");
    return ($? / 256);
}

