#!/usr/bin/perl
#
# $Id: raid.pl,v 1.8 2001/02/22 12:32:06 pbaltz Exp $
# checks RAID status, 
# updates Meta database and sends email to <sys@localhost>
#

# make sure /etc/raidtab exists
# present on RaQ4r but *NOT* on RaQ4i
#
exit 1 unless (-f "/etc/raidtab" );
# Watch out for StaQware
exit 1 if (-f "/etc/cobalt/ha-config" );

use Cobalt::Meta;
use Cobalt::RAID;
use Locale::gettext;
use lib qw( /usr/sausalito/perl );
use SendEmail;
use POSIX;

$MDSTAT='/proc/mdstat';
$dead=0;
$previous=0;

# setup locale information
#
$language=`cat /etc/cobalt/locale`;
$language="en_US" unless ($language);
setlocale(LC_MESSAGES, $language);
bindtextdomain("ui", "/usr/share/locale");
textdomain("ui");

# make sure /proc/mdstat exists
#
unless (-f $MDSTAT) {
    print STDERR "no $MDSTAT (kernel doesn't have md support?)\n";
    exit 7;
}
my $MDSTAT_data=`cat $MDSTAT`;

# query database to find out current alert status
#
$Obj = new Cobalt::Meta(type => defcon);
$Obj->retrieve(raid);
if (3 == $Obj->get(defcon)) {
    $previous=1;
}
$Obj->put(name => raid, special => disk);

my (@RAID_state) = raid_get_state();
# raid state 3 means disk failure
if ( 3 == $RAID_state[0] ) {
	system("/sbin/hdparm -b 2 /dev/$RAID_state[5] >& /dev/null");
	$dead=1;
}

if ($dead && !$previous) {
    # if raid error found
    # alert going from GREEN -> RED
    #
    $Obj->put(defcon => 3);
    $Obj->save;
    
		my $subject = gettext("raid.email.subject");
		my $errorlog = gettext("raid.failure.$RAID_state[5]");
		if ( ! ($RAID_state[5] eq 'hda' || $RAID_state[5] eq 'hdc') ) {
			$errorlog .= "\r\n\r\n" . Locale::gettext::dgettext('ui', "raid.sled.replacement");
		}
		my $techdetails = gettext("raid.email.techdetails");

    my $body =<<END;
$errorlog

$techdetails

[$MDSTAT]
$MDSTAT_data
END

SendEmail::sendEmail("System Alert List <sys\@localhost>", "Active Monitor <root\@localhost>", $subject, $body);

}
elsif ($previous && !$dead) {
    # no raid errors found
    # error state going from RED -> GREEN
    #
    $Obj->put(defcon => 1);
    $Obj->save;
}

# exit code is 0 for normal
# and 1 on error
#
exit $dead;
