# DNS autoconfig
# $Id: DNS.pm,v 1.10 2000/11/29 07:31:21 pbaltz Exp $
# Will DeHaan <will@cobalt.com>

package Cobalt::DNS;
use vars qw(@ISA @EXPORT @EXPORT_OK);
require Cobalt::UI::dns;
require Exporter;
require SelfLoader;
@ISA    = qw(Exporter SelfLoader);
@EXPORT = qw(dns_import dns_purge dns_migrate dns_remove);

use Cobalt::Product;
# $Dns_records, $DNS_autoconfig

use File::Copy;

use strict;

SelfLoader->load_stubs();

1;
__DATA__

sub dns_migrate
# Auto-transition named records through fqdn and ip changes
#
# Invoked by siteMod.pm
#
# docs
{
  my($oldhost, $newhost, $olddomain, $newdomain, $oldip, $newip) = @_;
  
  # return 0 unless (-e $DNS_autoconfig);
  sleep 2 if (-e "$Lockdir/records");
  sleep 4 if (-e "$Lockdir/records");

  open(LOCK, ">>$Lockdir/records");

  open(RECS, $Dns_records) || warn "Could not open: $Dns_records $!";
  while(<RECS>) {
    chop;
    @rec = split(/\s+/);

    $rec[1] = $newhost if ($rec[1] eq $oldhost);
    $rec[2] = $newdomain if ($rec[2] eq $olddomain);
    $rec[3] = $newip if ($rec[3] eq $oldip);
    print LOCK join(' ', @rec)."\n";
  }

  close(RECS);
  close(LOCK);

  move("$Lockdir/records", $Dns_records) || warn "Could not move $Lockdir/records to $Dns_records: $!";

  dns_rebuild();  
} 

sub dns_import
# Auto-populate named records on vsite operations
#
# Invoked by siteAdd.pm
#
# docs
{
  my($host, $domain, $ip, $www, $email) = @_;
 
  # return 0 unless (-e $DNS_autoconfig);
  sleep 2 if (-e "$Lockdir/records");
  sleep 4 if (-e "$Lockdir/records");

  # no MX record creation, what grief...
  $www = $www.' '.$email;
  my @www = split(/\s+/, $www);

  my(%new_records); # hash to avoid duplicate record creation with fqdn
                    # repetition between email and web aliases
  my(@rec); 

  open(LOCK, ">$Lockdir/records") || die "Could not write: $lockdir/records $!";
  open(OLDREC, $Dns_records) || warn "Could not open: $Dns_records $!";
  while(<OLDREC>) { 
    print LOCK;

    chop;
    @rec = split(/\s+/, $_);
    
    $new_records{"$rec[1]-$rec[2]"} = 1 if ($rec[0] eq 'a');
    $new_records{'mx-'.$rec[4]} = 1 if ($rec[0] eq 'mx');
  }
  close(OLDREC); # leave LOCK open for append...

  # fqdn
  print LOCK "a $host $domain $ip 24\n" unless ($new_records{"$host-$domain"});
  $new_records{"$host-$domain"} = 1; # remember the addition

  my $web;
  foreach $web (@www) {
    my $record_domain = $domain;

    if($web =~ /\.$domain$/) {
      $web =~ s/\.*$domain$//;
    } elsif ($web =~ /^([^\.]+)\.(.+\..+)$/) {
      ($web, $record_domain) = ($1, $2);
    } elsif ($web =~ /^([^\.]+)\.(.+)$/) {
      $record_domain = $web;
      $web = '';
    } else {
      next;
    }
     
    $web ||= '-';
    next unless ($record_domain =~ /$domain$/);

    print LOCK "a $web $record_domain $ip 24\n" unless ($new_records{"$web-$record_domain"});

    $new_records{"$web-$record_domain"} = 1; # remember
  }

  close(LOCK);

  move("$Lockdir/records", $Dns_records) || warn "Could not move $Lockdir/records to $Dns_records: $!";

  dns_rebuild();  
}

sub dns_purge
# Auto-depopulate named records on vsite operations
#
# Invoked by: siteDel.pm 
{
  my($host, $domain) = @_;

  # return 0 unless (-e $DNS_autoconfig);

  sleep 2 if (-e "$Lockdir/records");
  sleep 4 if (-e "$Lockdir/records");

  open(LOCK, ">>$Lockdir/records");

  open(RECS, $Dns_records);
  while(<RECS>) {
    @rec = split(/\s+/);
    print LOCK unless (($rec[0] =~ /^(a|mx)$/) && 
                       ($rec[2] eq $domain));
  }

  close(RECS);
  close(LOCK);

  move("$Lockdir/records", $Dns_records) || warn "Could not move $Lockdir/records to $Dns_records: $!";

  dns_rebuild();
}

sub dns_remove
# Depopulate named records
# Arguments: array of fqdn's 
#
# Invoked by: siteMod.pm
{
  my %fqdn = map {$_ => 1} @_;

  # return 0 unless (-e $DNS_autoconfig);

  sleep 2 if (-e "$Lockdir/records");
  sleep 4 if (-e "$Lockdir/records");

  open(LOCK, ">>$Lockdir/records");

  open(RECS, $Dns_records);
  while(<RECS>) {
    @rec = split(/\s+/);
    print LOCK unless ($fqdn{"$rec[1]\.$rec[2]"});
  }

  close(RECS);
  close(LOCK);

  move("$Lockdir/records", $Dns_records) || warn "Could not move $Lockdir/records to $Dns_records: $!";

  dns_rebuild();
}

###################################### unexported ###################3

sub dns_rebuild
{ 
  require(Cobalt::UI::dns);
  Cobalt::UI::dns::read_records();
  Cobalt::UI::dns::write_records();
  Cobalt::UI::dns::update_dns_config();
}

1;
