# Package for generating HTML 
#
# $Id: Html.pm,v 1.3 2000/11/29 07:31:21 pbaltz Exp $
#
# Based on Cobalt libPhp library
# !! Depends on images from 'gallery' and javascript from 'palette' modules.
#
# Copyright 2000 Cobalt Networks,Inc.
# http://www.cobalt.com

package Cobalt::Html;

use vars qw(@ISA @EXPORT);
require Exporter;
require SelfLoader;
@ISA    = qw(Exporter SelfLoader);
@EXPORT = qw(genSetSelector genSelectField);

SelfLoader->load_stubs();

1;
__DATA__

# Generates a pair of select lists that items can be toggled between.
# i.e. selecting an item and pressing a button will make the item jump
# to the other list.
# params:
# id: the HTML name identifier for the input field and the basis for the
#     names of the select fields: _$id_value, $id_entries
# values: array reference to "selected items", items belonging to the 
#         'value' select list
# entries: array reference to all possible values, items in entries but
#          not in values appear in the 'entries' list
# *label*: labels for the top or bottom of the respective select lists
# Returns: String of HTML for the two select lists, associated buttons,
#          and labels
sub genSetSelector {
	my ($id, $values, $entries, $valuelabel, $entrieslabel, $valuelabelbot, $entrieslabelbot) = @_;
	my $formId = "form";
	my $addId = "_".$id."_add";
	my $removeId = "_".$id."_remove";
	my $valueId = "_".$id."_value";
	my $entriesId = "_".$id."_entries";
	my ($addIcon, $removeIcon, $addIconDisabled, $removeIconDisabled) = ('/libImage/leftArrow.gif', '/libImage/rightArrow.gif', '/libImage/leftArrowGray.gif', '/libImage/rightArrowGray.gif');

	if ( ! @$values ) {
		@$values = ( '[str.selectListEmpty]' );
	}
	$valueSelect = genSelectField( $valueId, $formId, "top.code.SetSelector_setButtons(document.$formId.$id)", $values, $values );
	
	my @unselected;
	foreach ( @$entries ) {
		$keep = $_;
		if ( ! grep { /^$keep$/ } @$values ) {
			push @unselected, $keep;
		}
	}	
	if ( ! @unselected ) { 
		@unselected = ( '[str.selectListEmpty]' );
	}
	$entriesSelect = genSelectField( $entriesId, $formId, "top.code.SetSelector_setButtons(document.$formId.$id)", \@unselected, \@unselected );

	my $bottomlabel="";
	if ( $valuelabelbot || $entrieslabelbot) {
		$bottomlabel = "<TR><TD ALIGN=\"CENTER\" VALIGN=\"BOTTOM\">$valuelabelbot</TD><TD><IMG BORDER=\"0\" SRC=\"/libImage/spaceHolder.gif\" WIDTH=\"5\"></TD><TD ALIGN=\"CENTER\" VALIGN=\"BOTTOM\">$entrieslabelbot</TD></TR>";
	}

	my $result .= <<END;
<TABLE BORDER="0" CELLPADDING="0" CELLSPACING="0">
<TR><TD ALIGN="CENTER" VALIGN="BOTTOM">$valuelabel</TD><TD><IMG BORDER="0" SRC="/libImage/spaceHolder.gif" WIDTH="5"></TD><TD ALIGN="CENTER" VALIGN="BOTTOM">$entrieslabel</TD></TR>
  <TR>
    <TD ALIGN="CENTER">
      $valueSelect
    </TD>
    <TD ALIGN="CENTER" VALIGN="MIDDLE">
      <A HREF="javascript:top.code.SetSelector_moveItem(document.$formId.$id, 'add'); top.code.SetSelector_setButtons(document.$formId.$id)" onMouseOver="return true;"><IMG BORDER="0" NAME="$addId" SRC="$addIconDisabled"></A>
      <BR>
      <A HREF="javascript:top.code.SetSelector_moveItem(document.$formId.$id, 'remove'); top.code.SetSelector_setButtons(document.$formId.$id)" onMouseOver="return true;"><IMG BORDER="0" NAME="$removeId" SRC="$removeIconDisabled"></A>
    </TD>
    <TD ALIGN="CENTER">
      $entriesSelect
    </TD>
  </TR>
	$bottomlabel
</TABLE>

<INPUT TYPE="HIDDEN" NAME="$id" VALUE="">

<SCRIPT LANGUAGE="javascript">

var element = document.$formId.$id;
element.emptyLabel = "[str.selectListEmpty]";
element.focus = document.$formId.$valueId.focus;
element.parentDocument = document;

element.entriesElement = document.$formId.$entriesId;
element.valueElement = document.$formId.$valueId;

element.addButton = document.$addId;
element.addButton.url = "$addIcon";
element.addButton.disabledUrl = "$addIconDisabled";

element.removeButton = document.$removeId;
element.removeButton.url = "$removeIcon";
element.removeButton.disabledUrl = "$removeIconDisabled";

element.submitHandler = top.code.SetSelector_submitHandler;
element.isOptional = true;
</SCRIPT>

END

}


# Generates the HTML for a select list
# params: 
# id: string, the HTML name identifier for the input field
# formid: string, the id of the form this field is part of
# onChange: string, ONCHANGE field of the select list, typically
#           javascript to be executed when the list is touched
# labels: array reference to text shown for options in the list
# values: array reference to values associated with the labels, 
#         these are what get submitted
# selected: array reference to indices of selected items
# Returns: HTML for the select list
sub genSelectField {
	my ($id, $formid, $onChange, $labels, $values, $selected) = @_;
	my ($size, $width) = (6, 20);
	$onChange = ( $onChange ) ? "onChange=\"$onChange\"" : "";
	my $result = "<SELECT MULTIPLE NAME=\"$id\" ID=\"$id\" $onChange SIZE=\"$size\">\n";

	for ( $i=0; $i < @$labels; $i++ ) {
		$label = $$labels[$i];
		$value = $$values[$i];
		$selectstr = ( grep { /^$i$/ } @$selected ) ? "SELECTED" : "";

		$result .= "<OPTION VALUE=\"$value\" $selectstr>$label\n";
	}

	# add spacer
	$result .= "<OPTION>";
	for($i = 0; $i < $width; $i++) {
		$result .= "_";
	}

	$result .= "</SELECT>";
	$optionNum = @$labels;
	$result .= "<SCRIPT LANGUAGE = \"javascript\"> document.$formid.$id.options.length = $optionNum; </SCRIPT>";

	return $result;
}

1;
