#!/bin/sh
# Will DeHaan <will@cobalt.com> 
# $Id: Cobalt_Linux_unmanaged_conversion.sh,v 1.6 2000/10/27 15:17:45 will Exp $
#
# Disable all Cobalt management utilities leaving the user with an
# unmanaged Cobalt Linux box.  The conversion is one-way only, re-
# enabling Cobalt management services is not an option.  
#
# This utility performs the following:
# - Disables the administrative web interface
# - Derefrence the administrative web server from the public web server
# - Disables swatch daemon monitoring and repair
# - Disables scheduled Cobalt services, specifically backup
# - Terminates Usage statistics and report generation
# - Disables Postgresql server
# - Replaces Cobalt LCD console cross-configuration utilities
#   with simple resource monitoring, password reset and 
#   eth0 TCP/IP configuration
# - Deletes the Cobalt UI rpms, sauces remains intact
#
# THE UNMANAGED COBALT LINUX CONVERSION IS PERMANENT.
# IF YOU USE ANY COBALT ADMINISTRATIVE USER INTERFACES DO NOT 
# RUN THIS SCRIPT.  DO NOT EXPERIMENT WITH THIS UTILITY UNLESS
# YOU HAVE AN OS-RESTORE CD-ROM AND KNOW HOW TO USE IT.

ADMINIT=/etc/rc.d/init.d/admserv
ADMSERV=admserv
DOMAIN=special-sauce
HTTPDCNF=/etc/httpd/conf/httpd.conf
HTTPDINIT=/etc/rc.d/init.d/httpd
LCDSAVE=/etc/cobalt/lcd.stock/10main.m
LOG=/home/tmp/disable_Cobalt_services.log
NEWNETWORK=/usr/local/sbin/setup-network.whitebox
ORIGNETWORK=/usr/local/sbin/setup-network
POSTGRES=postgresql
POSTINIT=/etc/rc.d/init.d/postgresql
PROCESS=$$
TMPDIR=/home/tmp
WARNINGMSG=/etc/cobalt/messages/unmanaged_conversion_warning.txt
CCEINIT=/etc/rc.d/init.d/cced.init
CCED=cced.init

. /etc/rc.d/init.d/functions

clear

cat $WARNINGMSG

read bubba
if [ `echo $bubba | grep -E "(^Yes$|^YES$|^yes$)"` ]; then
  echo ""
  getmsg whitebox_proceeding
  sleep 2
else
  echo ""
  getmsg whitebox_declined
  echo ""
  exit 0
fi

# Initiate the log
echo `date` >> $LOG 2>&1

# Goodbye Admin UI
echo "" >> $LOG; echo "Stopping the administrative web server..." >> $LOG
/sbin/chkconfig --del $ADMSERV >> $LOG 2>&1
$ADMINIT stop >> $LOG 2>&1

# Goodbye CCE (Thank, God... :-) )
echo "" >> $LOG; echo "Stopping the administrative web server..." >> $LOG
/sbin/chkconfig --del $CCED >> $LOG 2>&1
$CCEINIT stop >> $LOG 2>&1


# Derefrence the administrative web server from the public web server 
echo "" >> $LOG; echo "Dereferencing admserv in httpd configuration..." >> $LOG
/bin/cp $HTTPDCNF $HTTPDCNF.stock
/usr/bin/perl -ne "if(/^<Virtual/.../^<\/Virtual/) { print; } else { print unless (/^RewriteCond/ ... /servername:81/); }" $HTTPDCNF > $TMPDIR/httpdcnf.$PROCESS 
/bin/mv $TMPDIR/httpdcnf.$PROCESS $HTTPDCNF >> $LOG
$HTTPDINIT restart >> $LOG 2>&1

# Deschedule swatch daemon monitoring utility
echo "" >> $LOG; echo "Descheduling swatch daemon monitoring..." >> $LOG
/bin/cp /etc/crontab /etc/crontab.stock >> $LOG 2>&1
/usr/bin/perl -pi -e "s#0,15,30,45 [^\n]+swatch#\# 0,15,30,45 \* \* \* \* root \/usr\/local\/sbin\/swatch#" /etc/crontab >> $LOG 2>&1

# Deschedule Cobalt backups
echo "" >> $LOG; echo "Descheduling Cobalt backup tasks..." >> $LOG
rm `find /etc -name "scheduled-backup.*"` >> $LOG 2>&1

# Terminate usage stats generation; this feels good..
echo "" >> $LOG; echo "Descheduling web, proftpd, email statistics generation..." >> $LOG
cp /etc/logrotate.d/apache /etc/cobalt/apache.logrotate
cp /etc/logrotate.d/proftpd /etc/cobalt/proftpd.logrotate
cp /etc/logrotate.d/syslog /etc/cobalt/syslog.logrotate
/usr/bin/perl -pi -e "s#/usr/local/sbin/split_logs#\# /usr/local/sbin/split_logs#" /etc/logrotate.d/apache >> $LOG 2>&1
/usr/bin/perl -pi -e "s#/usr/local/sbin/split_logs#\# /usr/local/sbin/split_logs#" /etc/logrotate.d/proftpd >> $LOG 2>&1
/usr/bin/perl -pi -e "s#/usr/local/sbin/split_logs#\# /usr/local/sbin/split_logs#" /etc/logrotate.d/syslog >> $LOG 2>&1

# Down postgresql
echo "" >> $LOG; echo "..." >> $LOG
/sbin/chkconfig --del $POSTGRES >> $LOG 2>&1
$POSTINIT stop >> $LOG 2>&1

# Strip & replace Cobalt LCD utils
echo "" >> $LOG; echo "Replacing Cobalt LCD console utilties..." >> $LOG
/bin/mv $ORIGNETWORK /usr/local/sbin/setup-network.stock >> $LOG 2>&1
/bin/ln -s $NEWNETWORK $ORIGNETWORK >> $LOG 2>&1
/bin/mkdir -p $LCDSTOCK >> $LOG 2>&1
/bin/mv /etc/lcd.d/10main.m/15CONFIGURE_UPS.s $LCDSTOCK/15CONFIGURE_UPS.s >> $LOG 2>&1 
/bin/mv /etc/lcd.d/10main.m/60SELECT_LANGUAGE.m $LCDSTOCK/60SELECT_LANGUAGE.m >> $LOG 2>&1

# disable reboot on LCD ip/nm/gw change
/usr/bin/perl -pi -e "s#system\(\"/sbin/reboot#\# system\(\"/sbin/reboot#" /etc/lcd.d/10main.m/10SETUP_NETWORK.m/10SETUP_NET1.m/10CONFIGURE.s/10ipnmgw >> $LOG 2>&1

# clear LCD console
/etc/rc.d/init.d/lcd-showip > /dev/null 2>&1

# delete the UI rpms, this is a unidirectional conversion folks
/bin/rpm -e --force --nodeps `/bin/rpm -qa |grep -G "^turbo_ui-"` >> $LOG 2>&1
/bin/rpm -e --force --nodeps `/bin/rpm -qa |grep -G "^ui-....raq"` >> $LOG 2>&1

# complete!
getmsg whitebox_completed



