#!/usr/bin/perl
# 
# Site Scheduled Backup
# Will DeHaan, will@cobalt.com/cobalt@guck.net
# (C) Cobalt Networks, Inc. http://cobalt.com/, 2000
#
# There's a reason that 1000+ employee backup-only companies exist.
# The work of one or two hosers can't do justice to disaster recovery.
# Lost web page?  Missing gifs?  Ok, can do. 
#
# So what's new?
# .tgz, personal raq backup headerless tar.gz format.  Simplicity.
#

use lib qw(/usr/sausalito/perl);
use strict;
use Cobalt::Vsite; 
use Cobalt::Product;
use File::Copy;
use vars qw( $AUTOLOAD );
use Carp;
use Net::FTP;
use POSIX;
use IPC::Open3;
use Cobalt::Util;

use Locale::gettext;

use SendEmail;

my $locale=`cat /etc/cobalt/locale`;
chomp $locale;

###############
# globals
my $invokee = $ARGV[0];

# globbing is bad here because we have to deal with the finite number
# of arguments imposed by the shell
# my $fileset = "* .*";

# set this here in case more directories under are allowed for backup
# in the future
my $fileset = 'web';  

###############
# bail without a config file
unless (-r $invokee) { 
	print <<EOF;
Cobalt Site Scheduled Backup
Configurable in the Site Settings administrative web interface under the "Backup" heading

Usage: site_backup.pl <path.to.gui.created.cron.job>

EOF
	exit;
}

###############
# who ami I?  Read config and assume permissions
my($user, $protocol, $remote_host, $remote_user, $remote_path, $remote_password);
my $user_code = 0;
open(SCHDL, $invokee) || die "Could not read schedule cron job $invokee as root: $!";
while(<SCHDL>) {
	chop;
                
	if($user_code) {
		next;
	} elsif(/^# USER: (.+)$/) {
		$user = $1;
	} elsif (/^# PROTOCOL: (.+)$/) { # either FTP NFS or SMB
		$protocol = $1;
	} elsif (/^# HOST: (.+)$/) { # simple fqdn
		$remote_host = $1;
	} elsif (/^# REMOTE_USER: (.+)$/) { # simple remote username
		$remote_user = $1;
	} elsif (/^# REMOTE_PATH: (.+)$/) { # path
		$remote_path = $1;
	} elsif (/^# PASSWORD: (.+)$/) { # path
		$remote_password = $1;
	} elsif (/below this line/) {
		$user_code = 1;
	}
}
close(SCHDL);
$remote_path ||= './';
my ($user_id, $user_dir) = (getpwnam($user))[2,7] if (($user =~ /^[a-z0-9][a-z0-9\.\-\_]+$/) && (length($user) < 32));
my $group = Cobalt::Vsite::vsite_getgroup($user);
unless($group) {
	$group ||= $invokee;
	$group =~ s/^.+\/([^\/]+)$/$1/;
}

my (@site_data) = Cobalt::Vsite::vsite_get_bygroup($group);
die "No site directory found, $Userdir/$group: $!" unless (-d "$Userdir/$group");
my $gid = (getgrnam($group))[2];

###############
# Setup environment
$ENV{'LANGUAGE'} = ($locale) ? $locale : 'en_US';
$ENV{'REMOTE_USER'} = $user; 
$ENV{'USER_FQDN'} = $site_data[1];

###############
# render filename
my($filename, $date_stamp);
chomp($date_stamp = `date +%m-%d-%Y`);
$filename = $site_data[1].'-'.$date_stamp.'.tgz'; # Concat so perl doesn't confuse $'s
my $backup_target = "$Big_tmp/$filename";
$backup_target = "$Big_tmp/dup-$filename" if (-e $backup_target);
&fin(0, '[str.backup.tmp.in.use.subject]', '[str.backup.tmp.in.use]') if (-e $backup_target);

###############
# become the spoon...
chdir("$Userdir/$group/");
# We can't su/gid since we need to (u)mount
# $( = $) = $gid; 
# $< = $> = $user_id;
umask(002);

###############
# setup our mounts
my $mount_pt = "/home/.sitebkp.$user.$$";
if ($protocol eq 'FTP') {
	unlink("$Userdir/$group/ncftp$$.cfg");

	my $ftp = Net::FTP->new("$remote_host");
	( ! defined($ftp) )
		? &fin(0, '[str.backup.ftp.no.connect.subject]', '[str.backup.ftp.no.connect]'.$@)
		: 0;
        
	if ($ftp && !$ftp->login($remote_user, $remote_password)) {
		&fin(0, '[str.backup.ftp.invalid.login.subject]', '[str.backup.ftp.invalid.login]');
	};
	
	( ! $ftp->cwd($remote_path) ) 
		? &fin(0, '[str.backup.ftp.invalid.path.subject]', '[str.backup.ftp.invalid.path]')
		: 0 if ($remote_path);

	open(NCFTP, "> $Userdir/$group/ncftp$$.cfg") || 
		&fin(0, '[str.backup.ftp.nfs.permissions.subject]', '[str.backup.ftp.nfs.permissions]');

	# ripped from Lovell's good code:
	printf (NCFTP "host %s\n", $remote_host);
	printf (NCFTP "user %s\n", $remote_user);
	printf (NCFTP "password %s\n", $remote_password);
	close(NCFTP);

} elsif ($protocol =~ /^(nfs|smb)$/i) {

	mkdir($mount_pt, 0700);
	&fin(0, '[str.backup.no.mountpoint.subject]', '[str.backup.no.mountpoint]') unless (-d $mount_pt);

} else {

	die "No defined backup protocol\n";
}

###############
# Bring on the tar
my $archive_report = `/bin/tar -czpPf $backup_target $fileset --exclude 'ncftp$$.cfg' --exclude '.' --exclude '..' 2>&1`
	&& die "could not open /bin/tar -czpPf $backup_target $fileset --exclude '.' --exclude '..': $!";

###############
# Transport data and/or clean up
my $err = 0;

if ($protocol eq 'FTP') { # FTP specific

	# ncftp our archive to any random remote server
	$archive_report .= "\n\n[str.backup.ftp.tranfser.log]\n";
	$archive_report .= `/usr/bin/ncftpput -f $Userdir/$group/ncftp$$.cfg -t 1 -V -m $remote_path $backup_target 2>&1`;

	if ($?) {
		$err = 1;
	}

	unlink($backup_target);
	unlink("$Userdir/$group/ncftp$$.cfg");

	if (not $err) {
		&fin(1, '[str.backup.ftp.good.subject]', '[str.backup.ftp.good]', '', $backup_target);  
	} else {
		&fin(0, '[str.backup.ftp.nfs.permissions.subject]', '[str.backup.ftp.nfs.permissions]', '', $backup_target);
	}

} elsif ($protocol eq 'SMB') {
	# figure out the actual share to mount
	$remote_path =~ /^([^\\]+)(.*)$/;
	my $smb_share = "\/\/$remote_host\/$1";
	my $dir = $2;

	# use an array instead of string, so command doesn't get passed through the shell
	my @smb_cmd = ("/bin/mount", '-t', 'smbfs', '-o');
	if ($remote_password) {
		push @smb_cmd, "rw,username=$remote_user,password=$remote_password";
	} else {
		push @smb_cmd, "rw,username=$remote_user";
	}
	
	push @smb_cmd, ($smb_share, $mount_pt); # , '>', '/dev/null', '2>&1');

	# my $smb_cmd = join ' ', @smb_cmd;
	# print STDERR "Attempting smb mount command:\n$smb_cmd\n\n";
	
	my @output = ();
	# try to mount the share
	{
		local (*WRITE, *READ);
		open3(\*WRITE, \*READ, '', @smb_cmd); # capture the output so cron only gets what we tell it
		while (<READ>) { push @output, $_; }
	}
	
	# ugh, mount backgrounds for smbfs if the first mount attempt fails
	# this should fix it for right now, but it may not be fool-proof
	&check_mount($mount_pt) || &fin(0, '[str.backup.nomount.smb.subject]', "[str.backup.nomount.smb]\n\n$!", $mount_pt, $backup_target);
	
	# figure out our target directory on the share
	my $location = "$mount_pt/$filename";

	$dir =~ s#(^\\|\\$)##g;
	$dir =~ s#\\#\/#g;
	
	if($dir) {
		$location = "$mount_pt/$dir/$filename";
	}

	copy($backup_target, $location) ||
	  &fin(0, '[str.backup.no.smb.copy.subject]', '[str.backup.no.smb.copy]', $mount_pt, $backup_target);

	&fin(1, '[str.backup.smb.good.subject]', '[str.backup.smb.good]', $mount_pt, $backup_target);

} elsif ($protocol eq 'NFS') {

	my $remote_share = $remote_host.$remote_path;

	my $nfs_cmd = "/bin/mount -t nfs -o rw $remote_share $mount_pt > /dev/null 2>&1";
	
	system($nfs_cmd) && &fin(0, '[str.backup.nomount.nfs.subject]', "[str.backup.nomount.nfs]\n\n$!", $mount_pt, $backup_target);

	copy($backup_target, "$mount_pt/$filename") ||
	  &fin(0, '[str.backup.no.nfs.copy.subject]', '[str.backup.no.nfs.copy]', $mount_pt, $backup_target);

	&fin(1, '[str.backup.nfs.good.subject]', '[str.backup.nfs.good]', $mount_pt, $backup_target);
}
	
	
###############
# No really, that's it folks
# FIXME: report in email using &fin()
exit 0;


########################### Local Subs ####################

sub email {
        # We use SendEmail.pm to handle the encoding
        # Unfortunately, I18n interpolate doesn't like multiple periods
        # in the i18n tags (it thinks the periods are part of a domain
        # identifier). So we have to do a gettext call on our own, using MSG_get 

        # FIXME. Ideally, we should just fix i18n so it handles multiple periods,
        # then just make this pass it's i18n tags onto SendEmail and trust
        # that SendEmail does the correct lookup.

	my($user, $subject, $body) = @_;
	my $from = "nobody@" . $ENV{'USER_FQDN'};
	my $disclaimer = "[str.backup.scheduled.disclaimer]";
	$body .= "\n\n$disclaimer\n";

	# interpolate body strings
	while($body =~ /\[str\.([^\]]+)\]/) {
		my $key = $1;
		my $message = MSG_get($key);
		$body =~ s/\[str\.$key\]/$message/g;
		undef($key, $message);
	}
	# substitute this variable into the disclaimer
	$body =~ s/\[SITEFQDN\]/$ENV{'USER_FQDN'}/g;

	# interpolate subject strings
	while($subject =~ /\[str\.([^\]]+)\]/) {
		my $key = $1;
		my $message = MSG_get($key);
		$subject =~ s/\[str\.$key\]/$message/g;
		undef($key, $message);
	}

	&SendEmail::sendEmail($user, $from, $subject, $body);
}

sub fin {
	my $success = shift;
	my $subject = shift;
	my $body = shift;
	my $mtpoint = shift;
	my $backup_target = shift;

	# unmount and delete the temporary mount point
	if ($mtpoint && (-d $mtpoint)) {
		my $ret = system("/bin/umount $mtpoint > /dev/null 2>&1");
		system("/bin/rm -rf $mtpoint > /dev/null 2>&1") unless ($ret);
	}

	# delete the local archive 
	unlink($backup_target) if (-e $backup_target);

	# add a file list garnish to the email report
	if($success) {
		$body .= "\n\n[str.backup.ftp.transfer.log]\n\n";
		my $ret = chdir('/home/sites/'.$ENV{'USER_FQDN'}.'/web');
		if ($ret) {
			$body .= '/home/sites/'.$ENV{'USER_FQDN'}."/web:\n";
			$body .= `/bin/ls -laR ./`; 
		}
	}

	# send an email report
	&email($ENV{'REMOTE_USER'}, $subject, $body);

	exit 0
}

sub check_mount {
	my $search = shift;

	open MOUNTS, '/etc/mtab' or die;

	while (<MOUNTS>) {
		if (/$search/) {
			return 1;
		}
	}

	close MOUNTS;
	return 0;
}
	
	
